"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isBoolean = void 0;
var Predicate_1 = require("./Predicate");
/**
 * @desc Ensures that the `value` is a {@link Boolean} value.
 *
 * @example
 * import { ensure, isBoolean, TinyType } from 'tiny-types';
 *
 * class MarketingOptIn extends TinyType {
 *     constructor(public readonly value: boolean) {
 *         ensure('MarketingOptIn', value, isBoolean());
 *     }
 * }
 *
 * @returns {Predicate<boolean>}
 */
function isBoolean() {
    return Predicate_1.Predicate.to("be a boolean value", function (value) { return typeof value === 'boolean'; });
}
exports.isBoolean = isBoolean;
//# sourceMappingURL=isBoolean.js.map