"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFunction = void 0;
var Predicate_1 = require("./Predicate");
/**
 * @desc Ensures that the `value` is a {@link Function}.
 *
 * @example
 * import { ensure, isFunction, TinyType } from 'tiny-types';
 *
 * function myFunction(callback: (error?: Error) => void): void {
 *     ensure('callback', callback, isFunction());
 * }
 *
 * @returns {Predicate<Function>}
 */
function isFunction() {
    return Predicate_1.Predicate.to("be a function", function (value) {
        return typeof value === 'function';
    });
}
exports.isFunction = isFunction;
//# sourceMappingURL=isFunction.js.map