"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isOneOf = void 0;
var isEqualTo_1 = require("./isEqualTo");
var or_1 = require("./or");
/**
 * @desc Ensures that the `value` is equal to one of the `allowedValues`
 *
 * @example
 * import { ensure, isOneOf, TinyType } from 'tiny-types';
 *
 * class StreetLight extends TinyType {
 *     constructor(public readonly value: string) {
 *         super();
 *
 *         ensure('StreetLight', value, isOneOf('red', 'yellow', 'green'));
 *     }
 * }
 *
 * @param {...T[]} allowedValues
 * @returns {Predicate<T>}
 */
function isOneOf() {
    var allowedValues = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        allowedValues[_i] = arguments[_i];
    }
    return or_1.or.apply(void 0, allowedValues.map(function (allowed) { return (0, isEqualTo_1.isEqualTo)(allowed); }));
}
exports.isOneOf = isOneOf;
//# sourceMappingURL=isOneOf.js.map