"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isString = void 0;
var Predicate_1 = require("./Predicate");
/**
 * @desc Ensures that the `value` is a  {@link String}.
 *
 * @example
 * import { ensure, isString, TinyType } from 'tiny-types';
 *
 * class FirstName extends TinyType {
 *     constructor(public readonly value: string) {
 *         ensure('FirstName', value, isString());
 *     }
 * }
 *
 * @returns {Predicate<string>}
 */
function isString() {
    return Predicate_1.Predicate.to("be a string", function (value) { return typeof value === 'string'; });
}
exports.isString = isString;
//# sourceMappingURL=isString.js.map